Option Explicit On
Option Strict On

Imports System
Imports System.DirectoryServices
Imports System.DirectoryServices.ActiveDirectory
Imports System.Text


Public Class CodeArticle



#Region "SID"

    ' Rcupre le SID d'un user dans un tableau d'octets
    Public Function getUserSid(ByVal cheminLdapUser As String, ByVal monUsername As String, ByVal monPassword As String) As Byte()

        ' Tableau d'octets qui va contenir le SID
        Dim sidUser As Byte() = Nothing

        Try

            ' Rcupration du SID de l'objet souhait
            Dim monEntry As New DirectoryEntry("LDAP://" & cheminLdapUser, monUsername, monPassword)
            sidUser = CType(monEntry.Properties("objectSid").Value, Byte())
            monEntry.Close()

        Catch ex As Exception

            MessageBox.Show(ex.Message)

        End Try

        ' Retour du SID dans un tableau d'octets sous sa forme primaire
        Return sidUser

    End Function

    ' Transforme le SID d'un tableau de Byte() en une chane de caractres
    Public Function formateSidToString(ByVal sidEnOctets As Byte()) As String

        Dim nombreSubauthority As Short = 0
        Dim monSid As StringBuilder = New StringBuilder

        monSid.Append("S-")

        Try
            monSid.Append(sidEnOctets(0).ToString)
            nombreSubauthority = Convert.ToInt16(sidEnOctets(1))


            If Not (sidEnOctets(2) = 0) OrElse Not (sidEnOctets(3) = 0) Then

                Dim strAuth As String = String.Format("0x{0:2x}{1:2x}{2:2x}{3:2x}{4:2x}{5:2x}", _
                                        CType(sidEnOctets(2), Int16), _
                                        CType(sidEnOctets(3), Int16), _
                                        CType(sidEnOctets(4), Int16), _
                                        CType(sidEnOctets(5), Int16), _
                                        CType(sidEnOctets(6), Int16), _
                                        CType(sidEnOctets(7), Int16))

                monSid.Append("-")
                monSid.Append(strAuth)

            Else

                Dim iVal As Int64 = CLng(CType((sidEnOctets(7)), Int32) & _
                                    CType((sidEnOctets(6) << 8), Int32) & _
                                    CType((sidEnOctets(5) << 16), Int32) & _
                                    CType((sidEnOctets(4) << 24), Int32))

                monSid.Append("-")
                monSid.Append(iVal.ToString)

            End If



            Dim idxAuth As Integer = 0
            Dim i As Integer = 0

            While i < nombreSubauthority

                idxAuth = 8 + i * 4

                Dim iSubAuth As UInt32 = BitConverter.ToUInt32(sidEnOctets, idxAuth)

                monSid.Append("-")
                monSid.Append(iSubAuth.ToString)

                i += 1

            End While


        Catch ex As Exception

            Return "Une erreur est survenue : " & ex.Message

        End Try


        MessageBox.Show(monSid.ToString)
        Return monSid.ToString

    End Function


#End Region





#Region "Utilisateurs et ordinateurs dans AD"


    ' Rcupration de la liste des users prsent dans un conteneur donn
    Public Function getListeUsersConteneur(ByVal cheminLdapConteneur As String, ByVal monUsername As String, ByVal monPassword As String) As ArrayList

        Dim listeUsers As New ArrayList()

        Try

            ' Connexion au conteneur souhait
            Dim monEntry As New DirectoryEntry("LDAP://" & cheminLdapConteneur, monUsername, monPassword, AuthenticationTypes.Secure)

            ' Paramtrage de la requte pour ne rcuprer que les objets "users"
            Dim maRequete As New DirectorySearcher(monEntry)
            maRequete.Filter = "((objectClass=user))"

            Dim uneEntry As DirectoryEntry
            Dim monResultat As SearchResult

            Dim nomUser As String = Nothing

            ' Rcupration de la proprit 'SAMAccountName' de chaque compte utilisateur
            For Each monResultat In maRequete.FindAll
                uneEntry = monResultat.GetDirectoryEntry
                nomUser = uneEntry.Properties("SAMAccountName").Value.ToString
                listeUsers.Add(nomUser)
            Next

        Catch ex As Exception

            MessageBox.Show(ex.Message)

        End Try

        Return listeUsers

    End Function

    ' Rcupration de la liste des ordinateurs prsent dans un conteneur donn
    Public Function getListeComputerContainer(ByVal conteneurAinspecter As String, ByVal monUsername As String, ByVal monPassword As String) As ArrayList

        ' Liste qui sera retourne avec les noms des ordinateurs 
        ' trouvs lors de la recherche
        Dim maListeComputers As New ArrayList

        Try

            ' Dfinition de la DirectoryEntry pour accder  l'annuaire
            Dim monEntry As DirectoryEntry = New DirectoryEntry("LDAP://" & conteneurAinspecter, monUsername, monPassword, AuthenticationTypes.Secure)
            Dim maRecherche As DirectorySearcher = New DirectorySearcher(monEntry)

            ' Filtre spcifiant que l'on recherche des objets de type 'computer'
            maRecherche.Filter = ("(objectClass=computer)")

            Dim unComputer As SearchResult = Nothing

            ' Boucle permettant de placer dans un liste chaque nom d'ordinateur
            ' retourn par la recherche.
            For Each unComputer In maRecherche.FindAll
                maListeComputers.Add(unComputer.GetDirectoryEntry.Name)
            Next

        Catch ex As Exception

            MessageBox.Show(ex.Message)

        End Try

        Return maListeComputers

    End Function

    ' Modification d'une proprits d'un user dans AD
    Public Function modifDescriptionUser(ByVal monCheminLdapUser As String, ByVal monUsername As String, ByVal monPassword As String, ByVal maValeur As String) As Boolean

        Try

            ' Connexion  l'objet utiliusateur souhait
            Dim monUser As DirectoryEntry = New DirectoryEntry("LDAP://" & monCheminLdapUser, monUsername, monPassword)
            ' Modification de la valeur
            monUser.Properties("description").Value = maValeur
            ' Validation des modifications
            monUser.CommitChanges()
            monUser.Close()

        Catch ex As Exception

            MessageBox.Show(ex.Message)
            Return False

        End Try

        Return True

    End Function

    ' Rcupration de la valeur de l'attribut 'displayName'
    Public Function getDisplayName(ByVal monCheminLdapUser As String, ByVal monUsername As String, ByVal monpassword As String) As String

        Dim monDisplayName As String = Nothing

        Try
            ' Connexion  l'objet compte utilisateur souhait
            Dim monUser As DirectoryEntry = New DirectoryEntry("LDAP://" & monCheminLdapUser, monUsername, monpassword)
            ' Rcupration de la valeur de la proprit
            monDisplayName = monUser.Properties("displayName").Value.ToString
            ' Fermeture de la "connexion"
            monUser.Close()

        Catch ex As Exception

            monDisplayName = ex.Message

        End Try

        Return monDisplayName

    End Function

    ' Rcupration du 'sAMAccountName' des utilisateurs prsent dans un conteneur ou une OU
    Public Function getUsersOU(ByVal monCheminLdapRecherche As String, ByVal monUsername As String, ByVal monpassword As String) As ArrayList


        ' ArrayList qui va contenir le rsultat retourn par la recherche
        Dim maListeUsers As New ArrayList

        Try

            ' Instanciation d'un objet DirectorySearcher

            ' Dfinition de l'emplacement de recherche
            Dim monEmplacementRecherche As New DirectoryEntry("LDAP://" & monCheminLdapRecherche, monUsername, monpassword, AuthenticationTypes.Secure)

            Dim maRecherche As New DirectorySearcher(monEmplacementRecherche)

            ' dureeMax initisalise  25 secondes
            Dim dureeMax As New TimeSpan(0, 0, 25)

            ' Emplacement o la recherche doit tre effectue 
            ' dans la hierarchie Active Directory
            maRecherche.SearchRoot = monEmplacementRecherche

            ' Dfinition du Scope de la recherche, ici le conteneur 
            ' seulement et tous ses "sous conteneur"
            maRecherche.SearchScope = SearchScope.Subtree

            ' Filtre uniquement les objets de type "user"
            maRecherche.Filter = "(objectClass=user)"

            ' Dtermination de la proprit  rcuprer lors de la recherche
            maRecherche.PropertiesToLoad.Add("sAMAccountName")

            ' Dure maximum de la recherche
            maRecherche.ServerTimeLimit = dureeMax

            ' Fixe le nombre maximum d'objets retourns
            maRecherche.SizeLimit = 1500

            Dim unUtilisateur As DirectoryServices.SearchResult

            ' Rcupration du 'sAMAccountName' des utilisateurs rcuprs
            For Each unUtilisateur In maRecherche.FindAll()
                maListeUsers.Add(unUtilisateur.GetDirectoryEntry.Properties.Item("sAMAccountName").Value.ToString)
            Next

            monEmplacementRecherche.Close()

        Catch ex As Exception
            MessageBox.Show(ex.Message)
        End Try

        Return maListeUsers

    End Function

    ' Ajoute un utilisateur  un groupe
    Public Function addUserToGroup(ByVal cheminLdapGroup As String, ByVal cheminLdapUser As String, ByVal monUsername As String, ByVal monPassword As String) As Boolean

        Dim monGroupe As New DirectoryEntry("LDAP://" & cheminLdapGroup, monUsername, monPassword, AuthenticationTypes.Secure)
        Dim monUtilisateur As New DirectoryEntry("LDAP://" & cheminLdapUser, monUsername, monPassword, AuthenticationTypes.Secure)

        Try
            ' Vrifie si l'utilisateur n'est pas dj membre du groupe
            Dim estDejaMembre As Boolean = Convert.ToBoolean(monGroupe.Invoke("IsMember", New Object() {monUtilisateur.Path}))

            If Not estDejaMembre Then
                ' Utilisation de la mthode ADSI "Add" pour ajouter un utilisateur  un groupe
                monGroupe.Invoke("Add", New Object() {monUtilisateur.Path})
            Else
                MessageBox.Show("L'utilisateur " & monUtilisateur.Properties("cn").Value.ToString() & " est dj membre de ce groupe !")
            End If

            Return True

        Catch ex As Exception

            MessageBox.Show(ex.Message)
            Return False

        Finally

            ' Libration des ressources inutiles encore utilises
            If Not IsNothing(monUtilisateur) Then
                monUtilisateur.Dispose()
            End If

            If Not IsNothing(monGroupe) Then
                monGroupe.Dispose()
            End If

        End Try

    End Function

    ' Retire un utilisateur d'un groupe
    Public Function removeUserToGroup(ByVal cheminLdapGroup As String, ByVal cheminLdapUser As String, ByVal monUsername As String, ByVal monPassword As String) As Boolean

        Dim monGroupe As New DirectoryEntry("LDAP://" & cheminLdapGroup, monUsername, monPassword, AuthenticationTypes.Secure)
        Dim monUtilisateur As New DirectoryEntry("LDAP://" & cheminLdapUser, monUsername, monPassword, AuthenticationTypes.Secure)

        Try
            ' Vrifie si l'utilisateur n'est pas dj membre du groupe
            Dim estMembre As Boolean = Convert.ToBoolean(monGroupe.Invoke("IsMember", New Object() {monUtilisateur.Path}))

            If estMembre Then
                ' Utilisation de la mthode ADSI "Add" pour ajouter un utilisateur  un groupe
                monGroupe.Invoke("Remove", New Object() {monUtilisateur.Path})
            Else
                MessageBox.Show("L'utilisateur " & monUtilisateur.Properties("cn").Value.ToString() & " n'est pas membre de ce groupe !")
            End If

            Return True

        Catch ex As Exception

            MessageBox.Show(ex.Message)
            Return False

        Finally

            ' Libration des ressources inutiles encore utilises
            If Not IsNothing(monUtilisateur) Then
                monUtilisateur.Dispose()
            End If

            If Not IsNothing(monGroupe) Then
                monGroupe.Dispose()
            End If

        End Try

    End Function

    ' Active un compte qui est dsactiv
    Public Function activeCompteUser(ByVal cheminLdapUser As String, ByVal monUsername As String, ByVal monPassword As String) As Boolean

        Try
            ' Connexion au compte utilisateur donn
            Dim monUtilisateur As DirectoryEntry = New DirectoryEntry("LDAP://" & cheminLdapUser, monUsername, monPassword, AuthenticationTypes.Secure)
            ' Rcupration du flag indiquant l'tat d'activation du compte
            Dim maValeur As Integer = CType(monUtilisateur.Properties("userAccountControl").Value, Integer)
            ' Modification du flag pour activer le compte
            monUtilisateur.Properties("userAccountControl").Value = maValeur And Not 2
            ' Validation des modifications
            monUtilisateur.CommitChanges()
            Return True

        Catch ex As Exception

            MessageBox.Show(ex.Message)
            Return False

        End Try


    End Function

    ' Desactive un compte activ
    Public Function desactiveCompteUser(ByVal cheminLdapUser As String, ByVal monUsername As String, ByVal monPassword As String) As Boolean

        Try
            ' Connexion au compte utilisateur donn
            Dim monUtilisateur As DirectoryEntry = New DirectoryEntry("LDAP://" & cheminLdapUser)
            ' Rcupration du flag indiquant l'tat d'activation du compte
            Dim maValeur As Integer = CType(monUtilisateur.Properties("userAccountControl").Value, Integer)
            ' Modification du flag pour dsactiver le compte
            monUtilisateur.Properties("userAccountControl").Value = maValeur Or 2
            ' Validation des modifications
            monUtilisateur.CommitChanges()
            Return True

        Catch ex As Exception

            MessageBox.Show(ex.Message)
            Return False

        End Try


    End Function

    ' Cration d'un objet compte d'utilisateur
    Public Function createUser(ByVal monCheminLdap As String, ByVal monUsername As String, ByVal monpassword As String, ByVal nouvelUser As String) As Boolean

        Try
            Dim monEmplacement As DirectoryEntry = New DirectoryEntry("LDAP://" & monCheminLdap, monUsername, monpassword, AuthenticationTypes.Secure)
            Dim monUser As DirectoryEntry = monEmplacement.Children.Add("cn=" & nouvelUser, "user")

            ' Nom d'ouverture de session pour les systmes antrieurs  Windows 2000
            monUser.Properties("sAMAccountName").Add("userDeveloppez")

            ' Nom affich dans l'annuaire
            monUser.Properties("displayName").Add("Mr Developpez")

            ' Adresse de l'utilisateur titulaire de ce compte
            monUser.Properties("street").Add("1 rue Developpez.com")

            ' Nom d'ouverture de session de l'utilisateur
            monUser.Properties("UserPrincipalName").Add("userDeveloppez@contoso.local")

            ' Description du compte utilisateur
            monUser.Properties("description").Add("Utilisateur de test")

            ' Cration de l'objet utilisateur et application des paramtres dfinis ci-dessus
            monUser.CommitChanges()

            'monUser.NativeObject.AccountDisabled = False

            ' Cration du mot de passe pour ce compte utilisateur
            monUser.Invoke("SetPassword", New Object() {"moTdePassE012"})
            monUser.CommitChanges()

            ' L'utilisateur devra changer son mot de passe  sa prochaine connexion
            monUser.Properties("pwdLastSet").Value = 0
            monUser.CommitChanges()

            Return True

        Catch ex As Exception

            MessageBox.Show(ex.Message)
            Return False

        End Try

    End Function

    ' Vrifie si un utilisateur donn existe ou non
    Public Function userExist(ByVal userAverifier As String, ByVal cheminLdapAexaminer As String, ByVal monUsername As String, ByVal monPassword As String) As Boolean

        Try
            ' Connexion  l'annuaire
            Dim monEntry As New DirectoryEntry(cheminLdapAexaminer, monUsername, monPassword, AuthenticationTypes.Secure)
            Dim maRecherche As DirectorySearcher = New DirectorySearcher

            ' Paramtrage de la requte
            maRecherche.SearchRoot = monEntry
            maRecherche.Filter = "(&(objectClass=user) (cn=" + userAverifier + "))"

            ' Rcupration du rsultat de la requte
            Dim results As SearchResultCollection = maRecherche.FindAll()
            monEntry.Close()

            ' Analyse du rsultat
            If results.Count = 0 Then
                Return False
            Else
                Return True
            End If

        Catch ex As Exception

            MessageBox.Show(ex.Message)

        End Try

    End Function

    ' Donne la liste des groupes dont un utilisateur est membre
    Public Function isMemberOf(ByVal cheminLdapUser As String, ByVal monUsername As String, ByVal monPassword As String) As ArrayList

        ' Instancie la liste qui va contenir les groupes
        Dim maListeGroupes As New ArrayList

        Dim monUser As New DirectoryEntry("LDAP://" & cheminLdapUser, monUsername, monPassword)

        ' Appel de la mthode "Groups" d'ADSI
        Dim mesGroupes As Object = monUser.Invoke("Groups")

        ' Rcupration de la liste des groupes
        For Each unGroupe As Object In CType(mesGroupes, IEnumerable)
            Dim groupEntry As DirectoryEntry = New DirectoryEntry(unGroupe)
            maListeGroupes.Add(groupEntry.Name)
            MessageBox.Show(groupEntry.Name)
        Next

        Return maListeGroupes

    End Function

    ' Donne la liste des membres d'un groupe
    Public Function listeMembresGroupe(ByVal cheminLdapGroupe As String, ByVal monUsername As String, ByVal monPassword As String) As ArrayList

        ' Instanciement de la liste qui va contenir le rsulat
        Dim maListeMembres As New ArrayList

        Try

            ' Groupe dont les membres sont  lister
            Dim monGroupe As New DirectoryEntry("LDAP://" & cheminLdapGroupe, monUsername, monPassword, AuthenticationTypes.Secure)

            Dim unMembre As New Object()
            ' Ajoute chaque membre trouv  notre ArrayList  retourner
            For Each unMembre In monGroupe.Properties("member")
                maListeMembres.Add(unMembre.ToString)
            Next

            monGroupe.Close()

        Catch ex As Exception

            MessageBox.Show(ex.Message)

        End Try

        ' Retourne la liste des membres du groupe
        Return maListeMembres

    End Function

    ' Supprime un comtpe utilisateur de l'annuaire
    Public Function deleteUser(ByVal cheminLdapParent As String, ByVal cheminLdapObjetAsupprimer As String, ByVal monUsername As String, ByVal monPassword As String) As Boolean

        ' Noeud parent de l'objet  supprimer
        Dim entryParent As New DirectoryServices.DirectoryEntry("LDAP://" & cheminLdapParent, monUsername, monPassword, AuthenticationTypes.Secure)
        ' Objet  supprimer
        Dim entryAsupprimer As New DirectoryServices.DirectoryEntry("LDAP://" & cheminLdapObjetAsupprimer, monUsername, monPassword, AuthenticationTypes.Secure)

        Try

            ' Suppression de l'objet
            entryParent.Children.Remove(entryAsupprimer)
            Return True

        Catch ex As Exception

            MessageBox.Show(ex.Message)
            Return False

        Finally

            ' Fermeture des DirectoryEntry
            entryParent.Close()
            entryAsupprimer.Close()

        End Try

    End Function

    ' Dplace un comtpe d'utilisateur dans l'annuaire
    Public Function moveUserFromTo(ByVal cheminLdapOrigine As String, ByVal cheminLdapDestination As String, ByVal monUsername As String, ByVal monPassword As String) As Boolean

        Try

            ' Connexion  l'utilisateur  dplacer
            Dim monUserAdeplacer As New DirectoryEntry("LDAP://" & cheminLdapOrigine, monUsername, monPassword, AuthenticationTypes.Secure)
            ' Dplacement de l'utilisareur vers son nouvel emplacement dans l'annuaire
            monUserAdeplacer.MoveTo(New DirectoryEntry("LDAP://" & cheminLdapDestination, monUsername, monPassword, AuthenticationTypes.Secure))

            monUserAdeplacer.Close()

            Return True

        Catch ex As Exception

            MessageBox.Show(ex.Message)
            Return False
        End Try

    End Function

    ' Cre une OU dans l'annuaire
    Public Function createOU(ByVal monCheminLdap As String, ByVal monUsername As String, ByVal monPassword As String, ByVal nouvelleOU As String) As Boolean
        Try

            ' Connexion au noeud de l'annuaire ou va tre ajoute l'OU
            Dim monEmplacement As DirectoryEntry = New DirectoryEntry("LDAP://" & monCheminLdap, monUsername, monPassword, AuthenticationTypes.Secure)
            ' Ajout de l'OU
            Dim monOU As DirectoryEntry = monEmplacement.Children.Add("OU=" & nouvelleOU, "organizationalUnit")
            ' Validation des modifications
            monOU.CommitChanges()

            Return True

        Catch ex As Exception

            MessageBox.Show(ex.Message)
            Return False

        End Try

    End Function

    ' Lister les OU
    Public Function listerOU(ByVal monCheminLdapRecherche As String, ByVal monUsername As String, ByVal monPassword As String) As ArrayList

        ' ArrayList qui va contenir le rsultat retourn par la recherche
        Dim maListeOU As New ArrayList

        Try


            ' Dfinition de l'emplacement de recherche
            Dim monEmplacementRecherche As New DirectoryEntry("LDAP://" & monCheminLdapRecherche, monUsername, monPassword, AuthenticationTypes.Secure)

            ' Instanciation d'un objet DirectorySearcher
            Dim maRecherche As New DirectorySearcher(monEmplacementRecherche)

            ' dureeMax initisalise  45 secondes
            Dim dureeMax As New TimeSpan(0, 0, 45)

            ' Emplacement o la recherche doit tre effectue 
            ' dans la hierarchie Active Directory
            maRecherche.SearchRoot = monEmplacementRecherche

            ' Dfinition du Scope de la recherche, ici le conteneur 
            ' seulement et tous ses "sous conteneur"
            maRecherche.SearchScope = SearchScope.Subtree

            ' Filtre uniquement les objets de type "organizationalUnit"
            maRecherche.Filter = "(objectClass=organizationalUnit)"

            ' Dtermination de la proprit  rcuprer lors de la recherche
            maRecherche.PropertiesToLoad.Add("name")

            ' Dure maximum de la recherche
            maRecherche.ServerTimeLimit = dureeMax

            ' Fixe le nombre maximum d'objets retourns
            maRecherche.SizeLimit = 1500

            Dim uneOU As DirectoryServices.SearchResult

            ' Rcupration du 'sAMAccountName' des utilisateurs rcuprs
            For Each uneOU In maRecherche.FindAll()
                maListeOU.Add(uneOU.GetDirectoryEntry.Properties.Item("name").Value.ToString)
                MessageBox.Show((uneOU.GetDirectoryEntry.Properties.Item("name").Value.ToString))
            Next

            monEmplacementRecherche.Close()

        Catch ex As Exception
            MessageBox.Show(ex.Message)
        End Try

        Return maListeOU

    End Function


#End Region






#Region "Informations infrastructure Active Directory"


    ' Rcupration de la liste des DC d'un domaine
    Public Function getListeDC() As DomainControllerCollection

        Dim listeDC As DomainControllerCollection = Nothing

        ' Dfinition du context de connexion : connexipn au domaine courant
        Dim monContext As New DirectoryContext(DirectoryContextType.Domain, Domain.GetCurrentDomain.ToString)
        Dim monDomaine As Domain = Domain.GetDomain(monContext)

        Try

            ' Rcupration de la liste des contrleurs de domaine
            listeDC = monDomaine.FindAllDomainControllers()

        Catch ex As Exception

            MessageBox.Show(ex.Message)

        End Try

        Return listeDC

    End Function

    ' Rcupration de la liste des CG d'un domaine
    Public Function getListeGC() As GlobalCatalogCollection

        Dim listeGC As GlobalCatalogCollection = Nothing

        Dim monContext As New DirectoryContext(DirectoryContextType.Forest, Forest.GetCurrentForest.ToString())
        Dim maForet As Forest = Forest.GetForest(monContext)

        Try

            listeGC = maForet.GlobalCatalogs()

            Dim unGC As GlobalCatalog
            For Each unGC In listeGC

                MessageBox.Show(unGC.Name.ToString())
            Next


        Catch ex As Exception

            MessageBox.Show(ex.Message)

        End Try


        Return listeGC

    End Function

    ' Rcupre le nom de la fort
    Private Function Getforest() As Forest

        Dim myForest As Forest = Nothing

        Try

            myForest = Forest.GetCurrentForest
            MessageBox.Show(myForest.Name.ToString())

        Catch ex As Exception

            MessageBox.Show(ex.Message)

        End Try


        Return myForest

    End Function

    ' Rcupration de la foret courante dans laquelle vous tes logu
    Public Function getNomForet() As String

        ' Dfinition d'une variable fort
        Dim maForet As Forest = Nothing

        Try

            ' Rcupration de la fort courante
            maForet = Forest.GetCurrentForest

        Catch ex As Exception

            MessageBox.Show(ex.Message)

        End Try

        Return maForet.ToString

    End Function

    ' Liste les domaines de la fret
    Public Function getDomainesForet() As DomainCollection

        ' Collection qui contiendra la liste des domaine
        Dim mesDomaines As DomainCollection = Nothing

        Try

            ' Rcupration de la liste des domaines de la fort courante
            mesDomaines = Forest.GetCurrentForest.Domains

        Catch ex As Exception

            MessageBox.Show(ex.Message)

        End Try

        Return mesDomaines

    End Function

    ' Rcupration du mode oprationnel d'un domaine
    Public Function getNiveauDomaine() As DomainMode

        ' Connexion au domaine courant
        Dim monContext As New DirectoryContext(DirectoryContextType.Domain, Domain.GetCurrentDomain.ToString)
        Dim monDomaine As Domain = Domain.GetDomain(monContext)
        Dim modeDomaine As DomainMode

        Try

            ' Rcupration du niveau du domaine
            modeDomaine = monDomaine.DomainMode

        Catch ex As Exception

            MessageBox.Show(ex.Message)

        End Try

        Return modeDomaine

    End Function

    ' Rcupration du niveau fonctionnel de la fort
    Public Function getNiveauForet() As ForestMode

        Dim monMode As ForestMode = Nothing

        Try
            ' Rcupration du niveau fonctionnel de la fort
            monMode = Forest.GetCurrentForest.ForestMode

        Catch ex As Exception

            MessageBox.Show(ex.Message)

        End Try

        Return monMode

    End Function

    ' Rcupration des titulaires des rles FSMO
    Public Function getListeRolesDC() As ArrayList

        Dim maListeRoles As New ArrayList
        Dim monDomaine As Domain = Domain.GetCurrentDomain

        Dim maitreInfrastructure As String = Nothing
        Dim maitreRID As String = Nothing
        Dim emulateurPDC As String = Nothing
        Dim maitreAttributionNoms As String = Nothing
        Dim controleurSchema As String = Nothing

        Try

            ' Matre d'infrastructure
            maitreInfrastructure = monDomaine.InfrastructureRoleOwner.ToString
            maListeRoles.Add("Matre d'infrastructure : " & maitreInfrastructure)

            ' Matre  RID
            maitreRID = monDomaine.RidRoleOwner.ToString
            maListeRoles.Add("Matre  RID : " & maitreRID)

            ' Emulateur PDC
            emulateurPDC = monDomaine.PdcRoleOwner.ToString
            maListeRoles.Add("Emulateur PDC : " & emulateurPDC)

            ' Matre d'attribution de noms de domaine 
            maitreAttributionNoms = monDomaine.Forest.NamingRoleOwner.ToString
            maListeRoles.Add("Matre d'attribution de noms de domaine : " & maitreAttributionNoms)

            ' Contrleur de schma
            controleurSchema = monDomaine.Forest.SchemaRoleOwner.ToString
            maListeRoles.Add("Contrleur de schma : " & controleurSchema)

        Catch ex As Exception

            MessageBox.Show(ex.Message)

        End Try

        Return maListeRoles

    End Function

    ' Rcupre la liste des appobations avec d'autres domaines
    Public Function getApprobationsDomaine() As TrustRelationshipInformationCollection

        Dim monDomaine As Domain = Domain.GetCurrentDomain
        Dim listeRelation As TrustRelationshipInformationCollection = Nothing

        Try

            listeRelation = monDomaine.GetAllTrustRelationships

        Catch ex As Exception

            MessageBox.Show(ex.Message)

        End Try

        Return listeRelation

    End Function

    ' Rcupre la liste des sites Active Directory de la fort courante
    Public Function getListeSitesDomaines() As ReadOnlySiteCollection

        Dim listeSites As ReadOnlySiteCollection = Nothing
        ' Connexion  la fort courante
        Dim monContext As New DirectoryContext(DirectoryContextType.Forest, Forest.GetCurrentForest.ToString)
        Dim maForet As Forest = Forest.GetForest(monContext)

        Try

            ' Rcupration de la liste des sites
            listeSites = maForet.Sites

        Catch ex As Exception

            MessageBox.Show(ex.Message)

        End Try

        Return listeSites

    End Function

    ' Cre un site Active Directory
    Public Function creationSite(ByVal nomSite As String) As Boolean

        Try

            Dim monContext As New DirectoryContext(DirectoryContextType.Forest, Forest.GetCurrentForest.ToString)
            Dim monSite As ActiveDirectorySite
            ' Cratio du nouveau site dans l'annuaire
            monSite = New ActiveDirectorySite(monContext, nomSite)
            ' Validation des modifications
            monSite.Save()

            Return True

        Catch ex As Exception

            MessageBox.Show(ex.Message)
            Return False

        End Try

    End Function

    ' Rcupre la liste des oprations de rplications qui n'ont pas t effectues 
    Public Function operationsReplicationNonEffectuees(ByVal nomDC As String) As ReplicationOperationCollection
        ' nomDC : nom DNS d'un contrleur de domaine

        Dim listeOperations As ReplicationOperationCollection = Nothing

        Try

            ' Connexion au contrleur de domaine spcifi en paramtre
            Dim monContext As New DirectoryContext(DirectoryContextType.DirectoryServer, nomDC)
            Dim monDC As DomainController = DomainController.GetDomainController(monContext)
            ' Rcupration des oprations de rplication non effectues
            listeOperations = monDC.GetReplicationOperationInformation.PendingOperations()

        Catch ex As Exception

            MessageBox.Show(ex.Message)

        End Try

        Return listeOperations

    End Function

    ' Initier la rplication d'une partition de l'annuaire
    Public Function initierReplication(ByVal nomDCorigine As String, _
                                       ByVal nomDCdestination As String, _
                                       ByVal partitionArepliquer As String) As Boolean

        ' nomDCorigine : nom DNS contrleur de domaine de 
        '       contenant la partition  rpliquer
        ' nomDCdestination : nom DNS du contrleur de domaine 
        '      o la partition doit tre rplique
        ' partitionArepliquer : nom distingu de la partition
        '       rpliquer

        Try

            Dim monContext As New DirectoryContext(DirectoryContextType.DirectoryServer, nomDCdestination)
            Dim monDC As DomainController = DomainController.GetDomainController(monContext)
            ' Initiation de la rplication
            monDC.SyncReplicaFromServer(partitionArepliquer, nomDCorigine)

            Return True

        Catch ex As Exception

            MessageBox.Show(ex.Message)
            Return False

        End Try

    End Function

    ' Transfert un rle FSMO donn d'un contrleur de domaine  un autre
    Private Function TransfertRoleFSMO(ByVal roleAtransferer As ActiveDirectoryRole, ByVal nomDCdestinationRole As String) As Boolean

        Try

            Dim monContext As New DirectoryContext(DirectoryContextType.DirectoryServer, nomDCdestinationRole)
            Dim monDC As DomainController = DomainController.GetDomainController(monContext)
            ' Transfert du rle vers le contrleur choisi
            monDC.TransferRoleOwnership(roleAtransferer)

        Catch ex As Exception

            MessageBox.Show(ex.Message)

        End Try

    End Function


#End Region


End Class
